import typing
import time
import re
import io

import httpx
from PIL import Image

from wombo.urls import urls, headers_gen, check_headers, auth_key_headers
from wombo.models import CreateTask, CheckTask
from wombo.base_dream import BaseDream


class Dream(BaseDream):
    def __init__(self) -> None:
        self.client = httpx.Client()

    def _get_js_filename(self) -> str:
        """Получает имя JS файла, откуда извлекаем Google Key"""

        response = self.client.get(urls["js_filename"])
        js_filename = re.findall(r"_app-(\w+)", response.text)

        return js_filename[0]

    def _get_google_key(self) -> str:
        """Получаем Google Key из JS файла"""
        js_filename = self._get_js_filename()

        url = f"https://dream.ai/_next/static/chunks/pages/_app-{js_filename}.js"
        response = self.client.get(url)

        key = re.findall(r'"(AI\w+)"', response.text)
        return key[0]

    def _get_auth_key(self) -> str:
        params = {"key": self._get_google_key()}
        json_data = {"returnSecureToken": True}

        response = self.client.post(
            urls["auth_key"],
            headers=auth_key_headers,
            params=params,
            json=json_data,
            timeout=20,
        )

        result = response.json()
        return result["idToken"]

    # ============================================================================================= #
    def create_task(self, text: str, style: int = 84) -> CreateTask:
        """We set the task to generate an image and use a certain TASK_ID, which we will track"""
        draw_url = "https://paint.api.wombo.ai/api/v2/tasks"
        auth_key = self._get_auth_key()
        data = (
            '{"is_premium":false,"input_spec":{"prompt":"%s","style":%d,"display_freq":10}}'
            % (text[:200], style)
        )

        response = self.client.post(
            draw_url, headers=headers_gen(auth_key), data=data.encode(), timeout=20
        )
        result = response.json()
        result = CreateTask.parse_obj(result)
        return result

    def check_task(self, task_id: str, only_bool: bool = True) -> CheckTask | bool:
        """Checks if the image has already been generated by task_id"""
        img_check_url = f"https://paint.api.wombo.ai/api/v2/tasks/{task_id}"

        response = self.client.get(img_check_url, headers=check_headers, timeout=10)
        result = CheckTask.parse_obj(response.json())
        return bool(result.photo_url_list) if only_bool else result

    def generate(self, text: str, style: int = 84, gif: bool = False):
        """Generate image"""
        task = self.create_task(text=text, style=style)
        time.sleep(2)
        for _ in range(100):
            task = self.check_task(task_id=task.id, only_bool=False)
            if task.photo_url_list and task.state != "generating":
                res = self.gif(task.photo_url_list) if gif else task
                break
            time.sleep(2)
        return res

    async def gif(self, url_list: typing.List) -> io.BytesIO:
        """Creating a streaming object with gif"""
        urls = [self.client.get(url) for url in url_list]
        frames = [Image.open(io.BytesIO(url.content)) for url in urls]
        return self.save_frames_as_gif(frames)


if __name__ == "__main__":
    dream = Dream()
    print(dream.check_task("22faeb1b-6adb-4210-9ac8-9289752d0a4a"))



"""
	URLs for calling APIs.
"""

from typing import Dict

urls = {
    "js_filename": "https://dream.ai/create",
    "auth_key": "https://identitytoolkit.googleapis.com/v1/accounts:signUp",
    "draw_url": "https://paint.api.wombo.ai/api/v2/tasks",
}

auth_key_headers = {
    "authority": "identitytoolkit.googleapis.com",
    "accept": "*/*",
    "accept-language": "ru,en;q=0.9",
    "content-type": "application/json",
    "origin": "https://dream.ai",
    "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "YaBrowser";v="23"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"Windows"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "cross-site",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
    "x-client-version": "Chrome/JsCore/9.1.2/FirebaseCore-web",
}

check_headers = {
    "authority": "paint.api.wombo.ai",
    "accept": "*/*",
    "accept-language": "ru,en;q=0.9",
    "authorization": "bearer eyJhbGciOiJSUzI1NiIsImtpZCI6ImM4MjNkMWE0MTg5ZjI3NThjYWI4NDQ4ZmQ0MTIwN2ViZGZhMjVlMzkiLCJ0eXAiOiJKV1QifQ.eyJwcm92aWRlcl9pZCI6ImFub255bW91cyIsImlzcyI6Imh0dHBzOi8vc2VjdXJldG9rZW4uZ29vZ2xlLmNvbS9wYWludC1wcm9kIiwiYXVkIjoicGFpbnQtcHJvZCIsImF1dGhfdGltZSI6MTY4MTY2NjM1MCwidXNlcl9pZCI6ImhUTTlGdmcxUjdURXJvd29FYnFBMU1ucG95QTIiLCJzdWIiOiJoVE05RnZnMVI3VEVyb3dvRWJxQTFNbnBveUEyIiwiaWF0IjoxNjgxNjY2MzUwLCJleHAiOjE2ODE2Njk5NTAsImZpcmViYXNlIjp7ImlkZW50aXRpZXMiOnt9LCJzaWduX2luX3Byb3ZpZGVyIjoiYW5vbnltb3VzIn19.cwcqHkKpVgq6B9CUnQmM4hA6uWmjW9jOdzC98MYZVvSgp02cn3lA9JUMNBqoZS3tKj2aLzIRRr0v-LRK4Hq-223vPI1USmDQ543BZZDlI4mGwR428hPOD8iIZxdR6d-kTEyvg82GtrJ7QIgTByrqkhAxwwdxMyLvEiUkAgggefjuTE0-dNxDxX9PI-_7bnJ5bc2sYFkwTiDQ_snyj34TE9C5stVtubtCqaCtKKEt3HO7nBhxRFjh7IeN18QWG8Qt1WLbKoqPFo3QJVuZDSk6tmb5YZpvwQjTsRuUjQSMTGytHMLBZ2eAubr7MQYctYklE8GDQ3G5dksgbVtCxsDI8g",
    "origin": "https://dream.ai",
    "referer": "https://dream.ai/",
    "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "YaBrowser";v="23"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"Windows"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "cross-site",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 YaBrowser/23.3.1.895 Yowser/2.5 Safari/537.36",
    "x-app-version": "WEB-1.90.1",
}


def headers_gen(auth_key: str) -> Dict:
    return {
        "authority": "paint.api.wombo.ai",
        "accept": "*/*",
        "accept-language": "ru,en;q=0.9",
        "authorization": f"bearer {auth_key}",
        "content-type": "text/plain;charset=UTF-8",
        "origin": "https://dream.ai",
        "referer": "https://dream.ai/",
        "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "YaBrowser";v="23"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "cross-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 YaBrowser/23.3.1.895 Yowser/2.5 Safari/537.36",
        "x-app-version": "WEB-1.90.1",
    }



from typing import List

from pydantic import BaseModel


class InputSpec(BaseModel):
    gen_type: str
    style: int
    prompt: str
    aspect_ratio_width: int
    aspect_ratio_height: int
    aspect_ratio: str


class Result(BaseModel):
    final: str


class CheckTask(BaseModel):
    id: str
    user_id: str
    state: str
    input_spec: InputSpec
    premium: bool
    created_at: str
    updated_at: str
    is_nsfw: bool
    photo_url_list: List[str]
    generated_photo_keys: List[str]
    result: Result = None



from typing import List
from io import BytesIO

from PIL.Image import Image
from httpx._client import BaseClient


class BaseDream:
    """
    Base class for all dreams (sync / async).
    """

    client: BaseClient

    def save_frames_as_gif(self, frames: List[Image], duration: int = 400) -> BytesIO:
        """
        Saves a list of frames as a gif and returns as bytes IO for writing stream.
        """
        result = BytesIO()

        frames[0].save(
            fp=result,
            save_all=True,
            append_images=frames[1:],
            format="GIF",
            duration=duration,
            loop=1,
        )

        return result
